open Jest;
open Expect;
open Lab79BsKnex2;
open TestUtil;
open Main;

module BaseModels = Lab79BsSmartPracticeModels;

beforeAllPromise(t =>
  Js.Promise.(
    generatePatient()
    |> then_(generatePatient)
    |> then_(generatePatient)
    |> then_(_ => Js.Promise.resolve(t))
  )
);

afterEachPromise(t =>
  Js.Promise.(
    batchCheckTasks(`OutboundCall |> BaseModels.Seed.taskChannelsToJs)
    |> then_(_ =>
         Knex2.(
           Model.knex
           |> BaseModels.TaskAttempts.Query.q
           |> del()
           |> whereIn("id", taskAttemptIds^ |> Array.of_list)
           |> toPromise
           |> then_(_ =>
                Model.knex
                |> BaseModels.Tasks.Query.q
                |> del()
                |> whereIn("id", taskIds^ |> Array.of_list)
                |> toPromise
              )
         )
         |> then_(_ => resolve(t))
       )
  )
);

afterAllPromise(t =>
  Js.Promise.(
    Knex2.(
      Model.knex
      |> fromTable("patients")
      |> del()
      |> whereIn("id", Array.of_list(patientIds^))
      |> toPromise
    )
    |> then_(_ => resolve(t))
  )
);

describe("Facts about `Task.Get.pendingByTaskTypeIdByChannel`", () =>
  testPromise("task ordering", _ =>
    Js.Promise.(
      /** 5 */
      generateTask(
        ~patientId=patientIds^ |> List.hd,
        ~state=`Dispatched,
        ~priority=0,
        ~earliest=((-2), `minutes),
        ~latest=(2, `minutes),
      )
      /*** 4 */
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds^ |> List.hd,
             ~state=`Dispatched,
             ~priority=0,
             ~earliest=((-5), `minutes),
             ~latest=(2, `minutes),
           )
         )
      /*** 3 */
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds^ |> List.hd,
             ~state=`Dispatched,
             ~priority=1,
             ~earliest=((-5), `minutes),
             ~latest=(2, `minutes),
           )
         )
      /*** 2 */
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds^ |> List.hd,
             ~state=`Dispatched,
             ~priority=1,
             ~earliest=((-2), `minutes),
             ~latest=(2, `minutes),
           )
         )
      /*** 1 */
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds^ |> List.hd,
             ~state=`Dispatched,
             ~priority=1,
             ~earliest=((-2), `minutes),
             ~latest=(2, `minutes),
           )
         )
      /*** 0 */
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds^ |> List.hd,
             ~priority=1,
             ~earliest=((-2), `minutes),
             ~latest=(2, `minutes),
             ~state=`Pending,
           )
         )
      |> then_(_ =>
           Model.Task.Get.queryDispatchedByChannelId(
             20,
             `OutboundCall |> BaseModels.Seed.taskChannelsToJs,
           )
         )
      |> then_(tasks => {
           let taskIds_ = taskIds^ |> Array.of_list;
           tasks
           |> Array.map(BaseModels.Tasks.Schema.idGet)
           |> expect
           |> toEqual([|
                Some(taskIds_[3]),
                Some(taskIds_[2]),
                Some(taskIds_[1]),
                Some(taskIds_[4]),
                Some(taskIds_[5]),
              |])
           |> resolve;
         })
    )
  )
);

describe("Rate limiting", () =>
  testPromise("count by target", () => {
    let patientIds_ = patientIds^ |> Array.of_list;

    Js.Promise.(
      generateTask(
        ~patientId=patientIds_[0],
        ~state=`Dispatched,
        ~priority=0,
        ~earliest=((-2), `minutes),
        ~latest=(2, `minutes),
      )
      |> then_(taskId =>
           generateTaskAttempt(
             ~taskId,
             ~channel=`OutboundCall,
             ~patientId=patientIds_[0],
           )
           |> then_(_ =>
                generateTaskAttempt(
                  ~taskId,
                  ~channel=`OutboundCall,
                  ~patientId=patientIds_[0],
                )
              )
         )
      |> then_(_ =>
           generateTask(
             ~patientId=patientIds_[1],
             ~state=`Dispatched,
             ~priority=0,
             ~earliest=((-2), `minutes),
             ~latest=(2, `minutes),
           )
           |> then_(taskId =>
                generateTaskAttempt(
                  ~taskId,
                  ~channel=`OutboundCall,
                  ~patientId=patientIds_[1],
                )
                |> then_(_ =>
                     generateTaskAttempt(
                       ~taskId,
                       ~channel=`OutboundCall,
                       ~patientId=patientIds_[1],
                     )
                   )
                |> then_(_ =>
                     generateTaskAttempt(
                       ~taskId,
                       ~channel=`OutboundCall,
                       ~patientId=patientIds_[1],
                     )
                   )
              )
         )
      |> then_(_ =>
           Model.TaskAttempt.CountByTarget.fromChannel(
             `OutboundCall |> BaseModels.Seed.taskChannelsToJs,
           )
         )
      |> then_(counts => {
           let expectedResult = Js.Dict.empty();
           expectedResult->Js.Dict.set(patientIds_[0], 2);
           expectedResult->Js.Dict.set(patientIds_[1], 3);
           counts |> expect |> toEqual(expectedResult) |> resolve;
         })
    );
  })
);