open Jest;
open Expect;

module BaseModels = Lab79BsSmartPracticeModels;

module Models = {
  [@bs.deriving abstract]
  type t('a) = {
    generateTestTask: 'a => Js.Promise.t(string),
    generateTestPatient: 'a => Js.Promise.t(string),
  };
};

[@bs.module "@lab-79/smart-practice-test-utils"]
external models: Models.t('a) = "models";

let models = models;

describe("dummy", () =>
  test("dummy", () =>
    expect(1) |> toBe(1)
  )
);

let patientIds = ref([]);
let taskIds = ref([]);
let taskAttemptIds = ref([]);
let generatePatient = () =>
  (models |> Models.generateTestPatientGet)([%bs.raw {|{}|}])
  |> Js.Promise.then_(id => {
       patientIds := [id, ...patientIds^];
       Js.Promise.resolve();
     });

let accountId = `DefaultAccount |> BaseModels.Seed.accountsToJs;

let generateTask = (~patientId, ~state, ~priority, ~earliest, ~latest) =>
  Model.knex
  |> BaseModels.Tasks.(
       Insert.new_(
         ~record=
           Schema.t(
             ~accountId,
             ~targetId=patientId,
             ~creatorType=`TaskRules,
             ~title="test task",
             ~creatorId=
               `TandignCVScheduleAppointment |> BaseModels.Seed.taskRulesToJs,
             ~taskTemplate=`AwvTaskTemplate,
             ~taskType=`ScheduleAppointment,
             ~targetType=`Patients,
             ~params=Js.Dict.empty(),
             ~priority,
             ~createdAt=Js.Date.fromFloat(Js.Date.now()),
             ~expectedEarliestAttempt=
               MomentRe.momentNow()
               |> MomentRe.Moment.add(
                    ~duration=
                      earliest->BaseModels.Util.RelTime.toMomentDuration,
                  )
               |> MomentRe.Moment.toDate,
             ~expectedLatestAttempt=
               MomentRe.momentNow()
               |> MomentRe.Moment.add(
                    ~duration=latest->BaseModels.Util.RelTime.toMomentDuration,
                  )
               |> MomentRe.Moment.toDate,
             ~expectedNextAttemptChannel=`OutboundCall,
             ~state,
             (),
           ),
       )
     )
  |> Js.Promise.then_(taskId => {
       taskIds := [taskId, ...taskIds^];
       Js.Promise.resolve(taskId);
     });

let generateTaskAttempt = (~taskId, ~channel, ~patientId) =>
  Model.knex
  |> BaseModels.TaskAttempts.Insert.fromTask(
       ~taskId,
       ~targetType=`Patients,
       ~targetId=patientId,
       ~channel,
     )
  |> Js.Promise.then_(taskAttemptId => {
       taskAttemptIds := [taskAttemptId, ...taskAttemptIds^];
       Js.Promise.resolve(taskAttemptId);
     });