open Lab79BsRequestPromiseNative;
module SP_Tasks = Lab79SmartPracticeTasks;
module BaseModels = Lab79BsSmartPracticeModels;

[@bs.val]
external appointmentSuggestionServiceUrl: string =
  "process.env.APPOINTMENT_SUGGESTION_SERVICE_URL";

module Physician = {
  module Query = {
    exception ApptSugServiceKeyNotExistForAccount;
    exception EcwDoctorIdMissing;
    exception EcwFacilityIdMissing;
    [@bs.deriving abstract]
    type t = {
      startDate: MomentRe.Moment.t,
      endDate: MomentRe.Moment.t,
      duration: int,
      ecwDoctorId: string,
      ecwFacilityId: string,
    };

    module Result = {
      module Future = {
        [@bs.deriving abstract]
        type t = {
          available: int,
          unavailable: int,
        };
      };

      [@bs.deriving abstract]
      type t = {
        past: int,
        future: Future.t,
      };
    };

    let check =
        (~accountId, ~startDate, ~endDate, ~duration, ~ecw)
        : Js.Promise.t(Result.t) =>
      switch (
        ecw
        |> SP_Tasks.ScheduleAppointmentTask.Params.Ecw.doctorIdGet
        |> Js.Json.decodeString
      ) {
      | None => raise(EcwDoctorIdMissing)
      | Some(ecwDoctorId) =>
        switch (
          ecw
          |> SP_Tasks.ScheduleAppointmentTask.Params.Ecw.facilityIdGet
          |> Js.Json.decodeString
        ) {
        | None => raise(EcwFacilityIdMissing)
        | Some(ecwFacilityId) =>
          BaseModels.Accounts.Get.byId(accountId, Model.knex)
          |> Js.Promise.then_(account =>
               switch (
                 account
                 |> BaseModels.Accounts.Get.appointmentSuggestionServiceAccessTokenGet
               ) {
               | None =>
                 Js.Promise.reject(ApptSugServiceKeyNotExistForAccount)
               | Some(token) =>
                 Request.request(
                   Request.RequestOpts.t(
                     ~uri=
                       appointmentSuggestionServiceUrl ++ "/availability_stats",
                     ~method="get",
                     ~auth=Request.auth(~user=accountId, ~pass=token),
                     ~qs=
                       t(
                         ~startDate,
                         ~endDate,
                         ~duration,
                         ~ecwDoctorId,
                         ~ecwFacilityId,
                       ),
                     ~body=None,
                     ~json=true,
                     (),
                   ),
                 )
               }
             )
        }
      };
  };
};