[@bs.module]
external knexfile: Js.Dict.t(Lab79BsKnex2Config.Config.KnexConf.t) =
  "../config/knexfile";

let knex =
  Lab79BsKnex2Config.DBConnection.getDB(knexfile, 2000)
  |> Lab79BsKnex2Config.DBConnection.knexGet;

open Lab79BsKnex2;
module BaseModels = Lab79BsSmartPracticeModels;

module Task = {
  module Update = {
    let resetState = (~taskId, ~toState, ~info) =>
      BaseModels.Tasks.(
        Update.byWhere(
          knex,
          ~target=Schema.t(~state=toState, ~stateInfo=info, ()),
          ~clause={"id": taskId},
        )
      );
  };

  module Get = {
    /**
         * fetch `batchSize` of tasks that has state of `"pending"` and
         * `earliest_attempt` is earlier than `NOW()` given a `taskType` and a `taskChannel`.
         * The more "old" a task is, (longest un-updated,
         *                            then longest not attempted before earliest attempt,
         *                            then the closest to deadline),
         * the higher pri will the task be checked
        */
    let queryDispatchedByChannelId =
        (batchSize, taskChannelId)
        : Js.Promise.t(array(BaseModels.Tasks.Schema.t)) =>
      Knex2.(
        knex
        |> BaseModels.Tasks.Query.q
        |> where({
             "expected_next_attempt_channel_id": taskChannelId,
             /*"task_type_id": taskTypeId,*/
             "state": "dispatched",
           })
        |> whereRaw("\"expected_earliest_attempt\" <= NOW()")
        |> orderBy2("priority", "DESC")
        |> orderBy2("expected_earliest_attempt", "ASC")
        |> orderBy2("expected_latest_attempt", "DESC")
        |> orderBy2("updated_at", "ASC")
        |> limit(batchSize)
        |> toPromise
      )
      |> Js.Promise.then_(tasks =>
           BaseModels.Tasks.Schema.Transform.allFromJs(tasks)
           ->Js.Promise.resolve
         );
  };
};

module TaskAttempt = {
  module CountByTarget = {
    [@bs.deriving abstract]
    type t_ = {
      [@bs.as "target_id"]
      targetId: string,
      count: string,
    };
    let fromChannel_ = (channelId: string): Js.Promise.t(array(t_)) =>
      Knex2.(
        knex
        |> fromTable("task_attempts")
        |> groupBy("target_id")
        |> select("target_id")
        |> Knex.count(Js.Nullable.return("id as count"))
        |> where({
             "state": `Procing |> BaseModels.Enums.taskAttemptStateToJs,
             "task_channel_id": channelId,
           })
      )
      |> Knex2.toPromise;
    let fromChannel = channelId =>
      fromChannel_(channelId)
      |> Js.Promise.then_(targetCounts => {
           let result = Js.Dict.empty();
           Array.iter(
             targetCount =>
               Js.Dict.set(
                 result,
                 targetCount |> targetIdGet,
                 targetCount |> countGet |> int_of_string,
               ),
             targetCounts,
           );
           result |> Js.Promise.resolve;
         });
  };
};